@@ -8,7 +8,7 @@
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -17,8 +17,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software Foundation,
- *   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -851,7 +850,7 @@ void mpop_retrmail_signal_handler(int signum UNUSED)
 
 int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 	account_t *acc, int debug, 
-	int quiet, int auth_only, int status_only,
+	int print_status, int print_progress, int auth_only, int status_only,
 	char **errmsg, char **errstr)
 {
     pop3_session_t *session;
@@ -873,6 +872,8 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
     int late_error;
     char *late_errmsg;
     char *late_errstr;
+    /* Whether QUIT was sent: */
+    int quit_was_sent = 0;
     
     
     /* create a new pop3_server_t */
@@ -986,6 +987,15 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 	}
     }
 #endif /* HAVE_TLS */
+    
+#if HAVE_SIGACTION
+    if (mpop_retrmail_abort)
+    {
+	mpop_endsession(session, 0);
+	*errstr = xasprintf(_("operation aborted"));
+	return EX_TEMPFAIL;
+    }
+#endif
 
     /* authenticate */
     if ((e = pop3_auth(session, acc->auth_mech, acc->username, 
@@ -1032,6 +1042,15 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 	    return exitcode_pop3(e);
 	}
     }
+    
+#if HAVE_SIGACTION
+    if (mpop_retrmail_abort)
+    {
+	mpop_endsession(session, 0);
+	*errstr = xasprintf(_("operation aborted"));
+	return EX_TEMPFAIL;
+    }
+#endif
 
     /* get status and scan listing */
     if ((e = pop3_stat(session, errmsg, errstr)) != POP3_EOK)
@@ -1041,7 +1060,11 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
     }
     if (session->total_number > 0)
     {
-	if ((e = pop3_list(session, errmsg, errstr)) != POP3_EOK)
+	if ((e = pop3_list(session, 
+#if HAVE_SIGACTION
+			&mpop_retrmail_abort, 
+#endif
+			errmsg, errstr)) != POP3_EOK)
 	{
 	    mpop_endsession(session, 0);
 	    return exitcode_pop3(e);
@@ -1065,7 +1088,11 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 	{
 	    /* the POP3 server does not support UIDL */
 	}
-	else if ((e = pop3_uidl(session, errmsg, errstr)) != POP3_EOK)
+	else if ((e = pop3_uidl(session,
+#if HAVE_SIGACTION
+			&mpop_retrmail_abort, 
+#endif
+			errmsg, errstr)) != POP3_EOK)
 	{
 	    if (e != POP3_EUNAVAIL)
 	    {
@@ -1144,7 +1171,7 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
     }
     
     /* Print status */
-    if (!quiet)
+    if (print_status)
     {
 	printf(_("%s at %s:\n"), acc->username, acc->host);
 	if (session->total_number > 0)
@@ -1214,7 +1241,7 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 	    if (acc->killsize >= 0 && session->msg_size[i - 1] >= acc->killsize)
 	    {
 		session->msg_action[i - 1] = POP3_MSG_ACTION_DELETE;
-		if (!quiet)
+		if (print_progress)
 		{
 		    if (acc->keep)
 		    {
@@ -1233,7 +1260,7 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 		    && session->msg_size[i - 1] >= acc->skipsize)
 	    {
 		session->msg_action[i - 1] = POP3_MSG_ACTION_IGNORE;
-		if (!quiet)
+		if (print_progress)
 		{
 		    printf(_("skipping message %ld of %ld (reason: "
     				"skipsize)\n"), i, session->total_number);
@@ -1276,7 +1303,8 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 #if HAVE_SIGACTION
 			&mpop_retrmail_abort, 
 #endif
-			acc->filter, quiet ? NULL : mpop_filter_output, acc,
+			acc->filter, 
+			print_progress ? mpop_filter_output : NULL, acc,
 			errmsg, errstr)) != POP3_EOK)
 	{
 	    if (uidl_list)
@@ -1288,20 +1316,6 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 	    return exitcode_pop3(e);
 	}
     }
-    
-#if HAVE_SIGACTION
-    if (mpop_retrmail_abort)
-    {
-	if (uidl_list)
-	{
-	    list_xfree(uidl_list, uidl_free);
-	}
-	fclose(uidls_fileptr);
-	mpop_endsession(session, 0);
-	*errstr = xasprintf(_("operation aborted"));
-	return EX_TEMPFAIL;
-    }
-#endif
 
     /* Once pop3_retr() is called, we cannot just abort the session and forget
      * everything we've done so far, because that would mean double mail
@@ -1310,7 +1324,7 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
     late_errmsg = NULL;
     late_errstr = NULL;
     
-    /* Retrieve */
+    /* Retrieve mails and update the UIDL accordingly */
     if (session->total_number > 0)
     {
 	late_error = pop3_retr(session, 
@@ -1318,54 +1332,85 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 		&mpop_retrmail_abort,
 #endif
 		acc->delivery_method, acc->delivery_args,
-		quiet ? NULL : mpop_retr_progress_start,
-		quiet ? NULL : mpop_retr_progress,
-		quiet ? NULL : mpop_retr_progress_end,
-	    	quiet ? NULL : mpop_retr_progress_abort,
+		print_progress ? mpop_retr_progress_start : NULL,
+		print_progress ? mpop_retr_progress : NULL,
+		print_progress ? mpop_retr_progress_end : NULL,
+	    	print_progress ? mpop_retr_progress_abort : NULL,
     		&late_errmsg, &late_errstr);
-    }
-    
-    /* Delete */
-    if (late_error == POP3_EOK && !acc->keep)
-    {
-	late_error = pop3_dele(session, &late_errmsg, &late_errstr);
-    }
-
-    /* Update the UIDL: only insert UIDs of messages that are retrieved and
-     * not deleted. */
-    for (i = 0; i < uidl->n; i++)
-    {
-	free(uidl->uidv[i]);
-    }
-    free(uidl->uidv);
-    uidl->uidv = NULL;
-    uidl->n = session->old_number;
-    if (uidl->n > 0)
-    {
-	uidl->uidv = xmalloc(uidl->n * sizeof(char *));
-	j = 0;
-	for (i = 0; i < session->total_number; i++)
+	for (i = 0; i < uidl->n; i++)
 	{
-	    if (session->is_old[i])
+	    free(uidl->uidv[i]);
+	}
+	free(uidl->uidv);
+	uidl->uidv = NULL;
+	uidl->n = session->old_number;
+	if (uidl->n > 0)
+	{
+	    uidl->uidv = xmalloc(uidl->n * sizeof(char *));
+	    j = 0;
+	    for (i = 0; i < session->total_number; i++)
 	    {
-		uidl->uidv[j++] = xstrdup(session->msg_uid[i]);
+		if (session->is_old[i])
+		{
+		    uidl->uidv[j++] = xstrdup(session->msg_uid[i]);
+		}
+	    }
+	}
+    }
+
+    /* Delete mails. If a failure occurs, save the current UIDL state. 
+     * Otherwise, end the session to commit the changes, then
+     * update the UIDL state and save it. */
+    if (late_error == POP3_EOK && !acc->keep)
+    {
+	if ((e = pop3_dele(session, 
+#if HAVE_SIGACTION
+		&mpop_retrmail_abort,
+#endif
+		errmsg, errstr)) != POP3_EOK)
+	{
+	    (void)uidls_write(acc->uidls_file, uidls_fileptr, uidl_list, NULL);
+	    list_xfree(uidl_list, uidl_free);
+    	    mpop_endsession(session, 0);
+	    return exitcode_uidls(e);
+	}
+	if ((e = pop3_quit(session, errmsg, errstr)) != POP3_EOK)
+	{
+	    (void)uidls_write(acc->uidls_file, uidls_fileptr, uidl_list, NULL);
+	    list_xfree(uidl_list, uidl_free);
+    	    mpop_endsession(session, 0);
+	    return exitcode_uidls(e);
+	}
+	quit_was_sent = 1;
+	for (i = 0; i < uidl->n; i++)
+	{
+	    free(uidl->uidv[i]);
+	}
+	free(uidl->uidv);
+	uidl->uidv = NULL;
+	uidl->n = session->old_number;
+	if (uidl->n > 0)
+	{
+	    uidl->uidv = xmalloc(uidl->n * sizeof(char *));
+	    j = 0;
+	    for (i = 0; i < session->total_number; i++)
+	    {
+		if (session->is_old[i])
+		{
+		    uidl->uidv[j++] = xstrdup(session->msg_uid[i]);
+		}
 	    }
 	}
     }
-    /* Save the updated UIDL information */
     if ((e = uidls_write(acc->uidls_file, uidls_fileptr, uidl_list, errstr)) 
 	    != UIDLS_EOK)
     {
-	free(late_errmsg);
-    	free(late_errstr);
 	list_xfree(uidl_list, uidl_free);
 	mpop_endsession(session, 0);
 	return exitcode_uidls(e);
     }
     list_xfree(uidl_list, uidl_free);
-
-    /* End session */
-    mpop_endsession(session, (late_error == POP3_EOK) ? 1 : 0);
+    mpop_endsession(session, late_error == POP3_EOK && !quit_was_sent);
 
     if (late_errmsg)
     {
@@ -1562,7 +1607,8 @@ int main(int argc, char *argv[])
     int print_conf;
     int debug;
     int pretend;
-    int quiet;
+    int print_status;
+    int print_progress;
     /* mode of operation */
     int retrmail;
     int serverinfo;
@@ -1609,6 +1655,7 @@ int main(int argc, char *argv[])
 	{ "version",         no_argument,       0, LONGONLYOPT_VERSION },
 	{ "help",            no_argument,       0, LONGONLYOPT_HELP },
 	{ "quiet",           no_argument,       0, 'q' },
+	{ "half-quiet",      no_argument,       0, 'Q' },
  	{ "pretend",         no_argument,       0, 'P' },
   	{ "debug",           no_argument,       0, 'd' },
 	{ "serverinfo",      no_argument,       0, 'S' },
@@ -1667,7 +1714,8 @@ int main(int argc, char *argv[])
     print_conf = 0;
     debug = 0;
     pretend = 0;
-    quiet = 0;
+    print_status = 1;
+    print_progress = 1;
     retrmail = 1;
     serverinfo = 0;
     all_accounts = 0;
@@ -1676,7 +1724,7 @@ int main(int argc, char *argv[])
     cmdline_account = account_new(NULL, NULL);
     for (;;)
     {
-	c = getopt_long(argc, argv, "qPdSC:aAsn::k::", options, NULL);
+	c = getopt_long(argc, argv, "qQPdSC:aAsn::k::", options, NULL);
 	if (c == -1)
 	{
 	    break;
@@ -1696,7 +1744,12 @@ int main(int argc, char *argv[])
 		break;
 
 	    case 'q':
-		quiet = 1;
+		print_status = 0;
+		print_progress = 0;
+		break;
+		
+	    case 'Q':
+		print_progress = 0;
 		break;
 		
   	    case 'P':
@@ -1707,9 +1760,8 @@ int main(int argc, char *argv[])
        	    case 'd':
 		print_conf = 1;
 	 	debug = 1;
-		/* normal output would interfere with
-		 * debugging output */
-		quiet = 1;
+		/* progress output would interfere with debugging output */
+		print_progress = 0;
 	 	break;
 
 	    case 'S':
@@ -2170,8 +2222,10 @@ int main(int argc, char *argv[])
 		"  --tls-force-sslv3[=(on|off)] Enable/disable restriction to "
 			"SSLv3.\n"
 	        "Options specific to mail retrieval mode:\n"
-		"  -q, --quiet                  Do not display progress "
-			"information.\n"
+		"  -q, --quiet                  Do not display status or "
+			"progress information.\n"
+		"  -Q, --half-quiet             Display status but not "
+			"progress information.\n"
 		"  -a, --all-accounts           Query all accounts in the "
 			"configuration file.\n"
 		"  -A, --auth-only              Authenticate only; do not "
@@ -2526,8 +2580,8 @@ int main(int argc, char *argv[])
 	    (void)sigaction(SIGINT, &signal_handler, &old_sigint_handler);
 #endif
 	    if ((error_code = mpop_retrmail(canonical_hostname, local_user,
-			    account, debug, quiet, auth_only, status_only,
-	    		    &errmsg, &errstr)) != EX_OK)
+			    account, debug, print_status, print_progress, 
+			    auth_only, status_only, &errmsg, &errstr)) != EX_OK)
 	    {
 		if (errstr)
 		{
