@@ -8,7 +8,7 @@
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -17,8 +17,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software Foundation,
- *   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef POP3_H
@@ -383,11 +382,17 @@ int pop3_stat(pop3_session_t *session, char **errmsg, char **errstr);
  * Issues the POP3 UIDL command without an argument.
  * This initializes the following fields of 'session':
  * - msg_uid
+ * If 'abort' is externally set, this function will abort and return
+ * POP3_EABORTED. The POP3 session is not usable thereafter.
  * Used error codes: POP3_EIO, POP3_EPROTO, POP3_EINVAL, POP3_EUNAVAIL.
  * The error POP3_EUNAVAIL is not critical: it means that the POP3 server does
  * not support the UIDL command.
  */
-int pop3_uidl(pop3_session_t *session, char **errmsg, char **errstr);
+int pop3_uidl(pop3_session_t *session,
+#ifdef HAVE_SIGACTION
+	volatile sig_atomic_t *abort,
+#endif
+	char **errmsg, char **errstr);
 
 /*
  * pop3_list()
@@ -395,9 +400,15 @@ int pop3_uidl(pop3_session_t *session, char **errmsg, char **errstr);
  * Issues the POP3 LIST command without an argument.
  * This initializes the following fields of 'session':
  * - msg_size
+ * If 'abort' is externally set, this function will abort and return
+ * POP3_EABORTED. The POP3 session is not usable thereafter.
  * Used error codes: POP3_EIO, POP3_EPROTO, POP3_EINVAL
  */
-int pop3_list(pop3_session_t *session, char **errmsg, char **errstr);
+int pop3_list(pop3_session_t *session,
+#ifdef HAVE_SIGACTION
+	volatile sig_atomic_t *abort,
+#endif
+	char **errmsg, char **errstr);
 
 /*
  * pop3_filter()
@@ -414,8 +425,8 @@ int pop3_list(pop3_session_t *session, char **errmsg, char **errstr);
  * If the action changed and filter_output is not NULL, the output function will
  * be called. 'data' can point to arbitrary user data that is passed to the
  * output function.
- * If 'abort' is externally set, this function will abort the filtering and
- * return POP3_EABORTED. The POP3 session is not usable thereafter.
+ * If 'abort' is externally set, this function will abort and return
+ * POP3_EABORTED. The POP3 session is not usable thereafter.
  * Used error codes: POP3_EIO, POP3_EPROTO, POP3_EINVAL, POP3_EDELIVERY,
  * POP3_EABORT.
  */
@@ -444,8 +455,8 @@ int pop3_filter(pop3_session_t *session,
  *    from 1 to 99
  * - 'progress_end' once, after everything was retrieved successfully
  * - 'progress_abort' only if an error occured, to clean up the output
- * If 'abort' is externally set, this function will abort the filtering and
- * return POP3_EABORTED. The POP3 session is not usable thereafter.
+ * If 'abort' is externally set, this function will abort and return
+ * POP3_EABORTED. The POP3 session is not usable thereafter.
  * Used error codes: POP3_EIO, POP3_EPROTO, POP3_EINVAL, POP3_EDELIVERY,
  * POP3_EABORT
  */
@@ -468,9 +479,15 @@ int pop3_retr(pop3_session_t *session,
  * POP3_MSG_ACTION_DELETE.
  * If it was set before, the is_old flag will be unset for each deleted message,
  * and old_number will be updated.
+ * If 'abort' is externally set, this function will abort and return
+ * POP3_EABORTED. The POP3 session is not usable thereafter.
  * Used error codes: POP3_EIO, POP3_EPROTO, POP3_EINVAL
  */
-int pop3_dele(pop3_session_t *session, char **errmsg, char **errstr);
+int pop3_dele(pop3_session_t *session, 
+#ifdef HAVE_SIGACTION
+	volatile sig_atomic_t *abort,
+#endif
+	char **errmsg, char **errstr);
     
 /*
  * pop3_rset()
