@@ -8,7 +8,7 @@
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -17,8 +17,7 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software Foundation,
- *   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -349,7 +348,10 @@ int uidls_write(const char *filename, FILE *uidls_file, list_t *uidl_list,
     if (fseek(uidls_file, 0, SEEK_SET) != 0 
 	    || ftruncate(fileno(uidls_file), 0) != 0)
     {
-	*errstr = xasprintf("%s: %s", filename, strerror(errno));
+	if (errstr)
+	{
+	    *errstr = xasprintf("%s: %s", filename, strerror(errno));
+	}
 	return UIDLS_EOK;
     }
 
@@ -384,7 +386,10 @@ int uidls_write(const char *filename, FILE *uidls_file, list_t *uidl_list,
 
     if (error)
     {
-	*errstr = xasprintf("%s: %s", filename, strerror(errno));
+	if (errstr)
+	{
+	    *errstr = xasprintf("%s: %s", filename, strerror(errno));
+	}
 	(void)fclose(uidls_file);
 	return UIDLS_EIO;
     }
